# parseMakeVars.tcl: parse variables from makefiles
#
# $Id: parseMakefile.tcl,v 1.11 2001/07/22 21:47:10 johnr Exp $
#
# Copyright (c) 1998-2001 The Regents of the University of California.
# All rights reserved. See the file COPYRIGHT for details.
#


### Assumed global variables:
# debug: print debug information if on
# silent: don't print anything if on


#######################################################################
#### parseMakefile
# Extract a makefile to get variables from it. This reads the file, looking
# for directives like include, and for variable definitions. Other
# gumpf, like comments and makerules, are ignored.
#
# For this to work, the current directory must be the same 
# as the makefile directory.
#
proc parseMakefile {filename} {
    global debug silent
    global _makeFileVars

    if $debug {
	puts "parseMakefile: [file join [pwd] $filename]"
    }

    # Open the file
    set fd [open $filename]

    # Set up regexp patterns.
    # A word is anything expect space or colon or percent sign
    set word "\[^ \t:%\]+"
    # Space is any white-space
    set space "\[ \t]*"
    # Space is some white-space
    set spaces "\[ \t]+"
    # stem is anything except space, colon, or period 
    set stem "\[^ \t:\]+"
    # pattern is anything with a prefix, suffix, and a percent in the middle
    set ppatt "$word%$word"

    # Read a line at a time
    while { ![eof $fd] } {
	set line [gets $fd]
	set foundrule 0

	# If the line ends with a backslash, continue it
	while { ![eof $fd] && [regexp {\\$} $line] } {
	    set line [string trimright [string trimright $line "\\"]]
	    append line " " [string trimleft [gets $fd]]
	}

	if [regexp "^$space#" $line] {
	    # Comment

	} elseif [regexp "^$space$" $line] {
	    # Blank

	} elseif [regexp "^${space}include${spaces}(.*)$" $line _ file] {
	    # Include another file
	    parseMakefile [string trim [substMakevars $file]]

	} elseif [regexp "^${space}(.*)${space}=${space}(.*)$" $line \
		_ name value] {
	    # Variable definition. Overwrite existing variables.
	    set name [string trim $name]
	    if [info exists _makeFileVars($name)] {
		if $debug {
		    puts "    Overwriting makefile variable \"$name\""
		}
	    }
	    set _makeFileVars($name) $value
	    if $debug {
		puts "    $name = $value"
	    }
	} elseif [regexp "^\.($stem)\.($stem)${space}:$space$" \
		_ dep tgt] {
	    # Suffix rule -- ignore
	    set foundrule 1

	} elseif [regexp "^(-\[^:\]*)${space}:$space\$" $line _ tgt] {
	    # An option-rule -- ignore
	    set foundrule 1

	} elseif { [regexp {%} $line] \
		&& [regexp "^(\[^:\]*)(:+)(\[^:\]*)$" $line \
		_ tgt colons dep] } {
	    # Pattern rule with implicit targets -- ignore
	    set foundrule 1

	} elseif { [regexp {%} $line] \
		&& [regexp "^(\[^:\]*):(\[^:\]*)(:+)(\[^:\]*)$" $line _ \
		lhs tgt colons dep] } {
	    # Pattern rule with explicit targets -- ignore

	} elseif { ![regexp {%} $line] \
		&& [regexp "^($word)${space}(:+)(\[^:\]*)$" $line _ \
		tgt colons dep] } {
	    # Simple rule -- ignore
	    set foundrule 1

	} else {
	    puts "    Unrecognized rule: $line"
	}
	# If we found a rule, read the command following it.
	# This is broken: if the line is a rule, then there is
	# no way to "unget" the line!
	if $foundrule {
	    #_printrule $id
	    set command {}
	    while { ![eof $fd] } {
		set line [gets $fd]

		# A command starts with space, and then has non-blank
		if [regexp "^\[ \t\]+\[^ \t\]" $line] {
		    append command $line\n
		} else {
		    break
		}
	    }
	    # Ignore
	    # set _command($id) $command
	}
    }
    close $fd
    if $debug {
	puts "EXIT parseMakefile: [file join [pwd] $filename]"
    }
}

#######################################################################
#### substMakevars
# Substitute make-file variables into a string.
# 
proc substMakevars {string} {
    global debug silent
    global _makeFileVars
    set varpatt "\\\$\\((\[^\)\]*)\\)"
    while { [regexp $varpatt $string _ varname] } {
	_catcherror {
	    set value [getMakevar $varname]
	}
	regsub -all {&} $value {___tclmake_ampersand___} value
	regsub $varpatt $string $value string
    }
    regsub -all {___tclmake_ampersand___} $string {\&} string
    return $string
}

#######################################################################
#### clearMakevars
# Clear all the makefile variables. This is useful to use if
# processing directories recursively.
# 
proc clearMakevars {} {
    global _makeFileVars
    catch {unset _makeFileVars}
}

#######################################################################
#### getMakevar
# Get the value of a makefile variable. This procedure calls itself
# recursively to find the final value. The first place it looks
# for a variable value is in the variable definitions read
# from the makefiles; if it doesn't find it, look in the
# environment. Finally, throw an error.
# 
proc getMakevar {varname} {
    global debug silent
    global _makeFileVars
    global env

    # Get its current value
    if ($debug) {
	puts "Getting $varname. Exists: [info exists _makeFileVars($varname)]"
    }
    if [info exists _makeFileVars($varname)] {
	set v $_makeFileVars($varname)
    } else {
	# Didn't find one -- try the environment
	if [info exists env($varname)] {
	    set _makeFileVars($varname) $env($varname)
	    set v $env($varname)
	} else {
	    error "Unknown variable: $varname"
	}
    }

    # If it contains variable references, resolve them
    set varpatt "\\\$\\((\[^\)\]*)\\)"
    while { [regexp $varpatt $v _ var] } {
	if { $var == $varname } {
	    error "Recursion in $varname = $v"
	}
	set value [getMakevar $var]
	regsub -all {&} $value {___tclmake_ampersand___} value
	regsub $varpatt $v $value v
    }
    regsub -all {___tclmake_ampersand___} $v {\&} v
    return $v
}

#######################################################################
#### checkMakevar
# Test to see if a variable is a known makefile variable.
# Does not look in the environment.
# 
proc checkMakevar {varname} {
    global _makeFileVars
    return [info exists _makeFileVars($varname)]
}








